<?php
/**
 * BSeller Platform | B2W - Companhia Digital
 *
 * Do not edit this file if you want to update this module for future new versions.
 *
 * @category  BSeller
 * @package   BSeller_SkyHub
 *
 * @copyright Copyright (c) 2018 B2W Digital - BSeller Platform. (http://www.bseller.com.br)
 *
 * @author    Tiago Sampaio <tiago.sampaio@e-smart.com.br>
 */

class BSeller_SkyHub_Model_Cron_Queue_Catalog_Category extends BSeller_SkyHub_Model_Cron_Queue_Abstract
{

    /**
     * @param Mage_Cron_Model_Schedule $schedule
     */
    public function create(Mage_Cron_Model_Schedule $schedule)
    {
        if (!$this->canRun($schedule)) {
            return;
        }

        $rootCategoryLevel = 1;

        /** @var Mage_Catalog_Model_Resource_Category_Collection $categories */
        $categories = $this->getCategoryCollection()
            ->addFieldToFilter('level', array('gt' => $rootCategoryLevel));

        if (!$categories->getSize()) {
            $schedule->setMessages($this->__('No category to be listed right now.'));
            return;
        }

        $categoryIds = array();

        /** @var Mage_Catalog_Model_Category $category */
        foreach ($categories as $category) {
            if ($category->isInRootCategoryList()) {
                continue;
            }

            $categoryIds[] = $category->getId();
        }
    
        $this->getQueueResource()->queue(
            $categoryIds,
            BSeller_SkyHub_Model_Entity::TYPE_CATALOG_CATEGORY,
            BSeller_SkyHub_Model_Queue::PROCESS_TYPE_EXPORT
        );

        $schedule->setMessages(
            $this->__('The categories were successfully queued. Category IDs: %s.', implode(',', $categoryIds))
        );
    }


    /**
     * @param Mage_Cron_Model_Schedule $schedule
     */
    public function execute(Mage_Cron_Model_Schedule $schedule)
    {
        $this->processStoreIteration($this, 'executeIntegration', $schedule);
    
        $successQueueIds = $this->extractResultSuccessIds($schedule);
        $failedQueueIds  = $this->extractResultFailIds($schedule);
    
        $this->getQueueResource()
             ->removeFromQueue($successQueueIds, BSeller_SkyHub_Model_Entity::TYPE_CATALOG_CATEGORY);
        
        $schedule->setMessages($this->__(
            'Queue was processed. Success: %s. Errors: %s.',
            implode(',', $successQueueIds),
            implode(',', $failedQueueIds)
        ));
    }
    
    
    /**
     * @param Mage_Cron_Model_Schedule $schedule
     */
    public function executeIntegration(Mage_Cron_Model_Schedule $schedule, Mage_Core_Model_Store $store)
    {
        if (!$this->canRun($schedule)) {
            return;
        }
    
        $categoryIds = (array) $this->getQueueResource()->getPendingEntityIds(
            BSeller_SkyHub_Model_Entity::TYPE_CATALOG_CATEGORY,
            BSeller_SkyHub_Model_Queue::PROCESS_TYPE_EXPORT
        );
    
        if (empty($categoryIds)) {
            $schedule->setMessages($this->__('No category to be integrated right now.'));
            return;
        }
    
        $successQueueIds = array();
        $failedQueueIds  = array();
    
        /** @var Mage_Catalog_Model_Category $category */
        foreach ($categoryIds as $categoryId) {
            /** @var Mage_Catalog_Model_Category $category */
            $category = $this->getCategory($categoryId, $store);
            
            /** @var \SkyHub\Api\Handler\Response\HandlerInterface $response */
            $response = $this->catalogCategoryIntegrator()->createOrUpdate($category);
        
            if ($this->isErrorResponse($response)) {
                $failedQueueIds[] = $categoryId;
            
                $this->getQueueResource()->setFailedEntityIds(
                    $categoryId,
                    BSeller_SkyHub_Model_Entity::TYPE_CATALOG_CATEGORY,
                    $response->message()
                );
            
                continue;
            }
    
            $successQueueIds[] = $categoryId;
        }
    
        $this->mergeResults($schedule, $successQueueIds, $failedQueueIds);
    }
    
    
    /**
     * @return Mage_Catalog_Model_Category
     */
    protected function getCategory($categoryId, Mage_Core_Model_Store $store = null)
    {
        $data = array('disable_flat' => true);
        
        /** @var Mage_Catalog_Model_Category $category */
        $category = Mage::getModel('catalog/category', $data);
        $category->setStoreId($store->getId())
                 ->load($categoryId);
        
        return $category;
    }


    /**
     * @return Mage_Catalog_Model_Resource_Category_Collection
     */
    protected function getCategoryCollection()
    {
        /** @var Mage_Catalog_Model_Resource_Category_Collection $collection */
        $collection = Mage::getResourceModel('catalog/category_collection');
        $collection->setDisableFlat(true);
        
        return $collection;
    }


    /**
     * @param array $categoryIds
     *
     * @return mixed
     *
     * @throws Mage_Core_Model_Store_Exception
     */
    protected function removeRootCategory(array &$categoryIds)
    {
        foreach ($categoryIds as $key => $categoryId) {
            if ($categoryId == Mage::app()->getStore()->getRootCategoryId()) {
                unset($categoryIds[$key]);
            }
        }

        return $categoryIds;
    }


    /**
     * @param Mage_Cron_Model_Schedule $schedule
     * @param int|null                 $storeId
     *
     * @return bool
     */
    protected function canRun(Mage_Cron_Model_Schedule $schedule, $storeId = null)
    {
        if (!$this->getCronConfig()->catalogCategory()->isEnabled($storeId)) {
            $schedule->setMessages($this->__('Catalog Category Cron is Disabled'));
            return false;
        }

        return parent::canRun($schedule, $storeId);
    }
}
